@@ -23,7 +23,7 @@ MeasureOOBError = R6Class("MeasureOOBError",
         id = "oob_error",
         task_type = NA_character_,
         properties = c("na_score", "requires_learner"),
-        predict_type = "response",
+        predict_type = NA_character_,
         range = c(-Inf, Inf),
         minimize = TRUE,
         label = "Out-of-bag Error",
