@@ -44,20 +44,6 @@ test_that("NA predictions", {
   expect_equal(is.na(p$response), apply(p$prob, 1, anyMissing))
 })
 
-test_that("test set is available in $.train method", {
-  task = tsk("iris")
-  learner = lrn("classif.debug", save_tasks = TRUE)
-  resampling = rsmp("cv", folds = 3)
-  resampling$instantiate(task)
-
-  rr = resample(task, learner, resampling, store_models = TRUE)
-
-  walk(seq(rr$iters), function(i) {
-    expect_equal(rr$learners[[i]]$model$task_train$row_roles$use, resampling$train_set(i))
-    expect_equal(rr$learners[[i]]$model$task_train$row_roles$test, resampling$test_set(i))
-  })
-})
-
 test_that("default_values", {
   learner = lrn("classif.debug")
   search_space = ps(iter = p_int(1, 10))
@@ -72,6 +58,20 @@ test_that("default_values works with empty search space", {
   expect_list(default_values(learner, ps(), task), len = 0)
 })
 
+test_that("validation and internal tuning", {
+  task = tsk("iris")
+  learner = lrn("classif.debug", iter = 100, early_stopping = TRUE, validate = 0.3, predict_type = "prob")
+  learner$train(task)
+  expect_list(learner$internal_valid_scores, len = 2L, types = "numeric")
+  expect_permutation(names(learner$internal_valid_scores), c("acc", "mbrier"))
+})
+
+test_that("disable internal tuning", {
+  learner = lrn("classif.debug", early_stopping = TRUE, iter = 100, validate = 0.2)
+  learner$param_set$disable_internal_tuning("iter")
+  expect_false(learner$param_set$values$early_stopping)
+})
+
 test_that("marshaling", {
   l = lrn("classif.debug")
   expect_learner(l, tsk("iris"))
